package fip.model;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.TreeMap;

import fip.model.exceptions.BadMessageId;

/**
 * Facade d'accès au forum.
 * <p> Anciennement, implémenté comme un singleton. Dans la version actuelle : bean application.
 * <p>Objet partagée par tous les threads. Attention aux problèmes
 * de synchronisation <strong>(ne pas utiliser cette approche dans une vraie application.)</strong>
 * <p> L'implémentation actuelle est très très inefficace (on bloque sur tout accès avec synchronized)
 * @author rosmord
 */
public class ForumFacade implements Serializable {

   

    /**
     * La clef maximale (thread-safe.)
     */
    private int maxKey = 0;

    /**
     * La map des messages.
     * On peut envisager aussi ici une ConcurrentHashMap, mais 
     * à la réflexion, on va rester simple, d'autant qu'on est plus ou moins obligé d'utiliser
     * synchronized de toute manière.
     */
    private TreeMap<Integer, Message> messagesMap = new TreeMap<>();
    
    
    /**
     * Ajoute un message.
     * 
     * @param titre
     * @param contenu
     * @return l'identifiant du nouveau message.
     */
    public synchronized int addMessage(String titre, String contenu) {
        maxKey++;
        int idx = maxKey;
        Message message = new Message(idx, titre, contenu);
        messagesMap.put(idx, message);
        return idx;
    }

    /**
     * Retourne une copie de la liste de tous les messages.
     * On n'a pas besoin de recopier les messages individuels, car ils sont immuables !!
     * @return
     */
    public synchronized List<Message> getMessageList() {
        return new ArrayList<>(messagesMap.values());
    }

    public synchronized Message getMessage(int i) {
        return messagesMap.get(i);
    }

    /**
     * Met à jour un message existant.
     * Le message doit correspondre à un message déjà présent, 
     * avec le même id.
     * @param nouvelleValeur : nouvelle valeur du message.
     * @throws BadMessageId si aucun message avec cet id n'est présent.
     */
    public synchronized void updateMessage(Message nouvelleValeur) throws BadMessageId{
        int id= nouvelleValeur.getId();
        if (messagesMap.containsKey(id)) {        
            messagesMap.put(id, nouvelleValeur);
        } else {
            throw new BadMessageId(id);
        }
    }

}
